
import cython


cdef cython.uint DNS_COMPRESSION_HEADER_LEN
cdef cython.uint MAX_DNS_LABELS
cdef cython.uint DNS_COMPRESSION_POINTER_LEN
cdef cython.uint MAX_NAME_LENGTH

cdef object current_time_millis

cdef cython.uint _TYPE_A
cdef cython.uint _TYPE_CNAME
cdef cython.uint _TYPE_PTR
cdef cython.uint _TYPE_TXT
cdef cython.uint _TYPE_SRV
cdef cython.uint _TYPE_HINFO
cdef cython.uint _TYPE_AAAA
cdef cython.uint _TYPE_NSEC
cdef cython.uint _FLAGS_QR_MASK
cdef cython.uint _FLAGS_QR_MASK
cdef cython.uint _FLAGS_TC
cdef cython.uint _FLAGS_QR_QUERY
cdef cython.uint _FLAGS_QR_RESPONSE

cdef object UNPACK_3H
cdef object UNPACK_6H
cdef object UNPACK_HH
cdef object UNPACK_HHiH

cdef object DECODE_EXCEPTIONS

cdef object IncomingDecodeError

from .._dns cimport (
    DNSAddress,
    DNSEntry,
    DNSHinfo,
    DNSNsec,
    DNSPointer,
    DNSQuestion,
    DNSRecord,
    DNSService,
    DNSText,
)


cdef class DNSIncoming:

    cdef bint _did_read_others
    cdef public unsigned int flags
    cdef cython.uint offset
    cdef public bytes data
    cdef unsigned int _data_len
    cdef public cython.dict name_cache
    cdef public cython.list questions
    cdef cython.list _answers
    cdef public object id
    cdef public cython.uint num_questions
    cdef public cython.uint num_answers
    cdef public cython.uint num_authorities
    cdef public cython.uint num_additionals
    cdef public object valid
    cdef public object now
    cdef public object scope_id
    cdef public object source

    @cython.locals(
        off=cython.uint,
        label_idx=cython.uint,
        length=cython.uint,
        link=cython.uint,
        link_data=cython.uint
    )
    cdef _decode_labels_at_offset(self, unsigned int off, cython.list labels, cython.set seen_pointers)

    cdef _read_header(self)

    cdef _initial_parse(self)

    @cython.locals(
        end=cython.uint,
        length=cython.uint
    )
    cdef _read_others(self)

    cdef _read_questions(self)

    cdef bytes _read_character_string(self)

    cdef _read_string(self, unsigned int length)

    @cython.locals(
        name_start=cython.uint
    )
    cdef _read_record(self, object domain, unsigned int type_, object class_, object ttl, unsigned int length)

    @cython.locals(
        offset=cython.uint,
        offset_plus_one=cython.uint,
        offset_plus_two=cython.uint,
        window=cython.uint,
        bit=cython.uint,
        byte=cython.uint,
        i=cython.uint,
        bitmap_length=cython.uint,
    )
    cdef _read_bitmap(self, unsigned int end)

    cdef _read_name(self)



import cython

from ._dns cimport (
    DNSAddress,
    DNSEntry,
    DNSHinfo,
    DNSPointer,
    DNSRecord,
    DNSService,
    DNSText,
)


cdef object _UNIQUE_RECORD_TYPES
cdef object _TYPE_PTR

cdef _remove_key(cython.dict cache, object key, DNSRecord record)


cdef class DNSCache:

    cdef public cython.dict cache
    cdef public cython.dict service_cache

    cdef _async_add(self, DNSRecord record)

    cdef _async_remove(self, DNSRecord record)


cdef _dns_record_matches(DNSRecord record, object key, object type_, object class_)




import cython

from .incoming cimport DNSIncoming


cdef cython.uint _CLASS_UNIQUE
cdef cython.uint _DNS_PACKET_HEADER_LEN
cdef cython.uint _FLAGS_QR_MASK
cdef cython.uint _FLAGS_QR_QUERY
cdef cython.uint _FLAGS_QR_RESPONSE
cdef cython.uint _FLAGS_TC
cdef cython.uint _MAX_MSG_ABSOLUTE
cdef cython.uint _MAX_MSG_TYPICAL


cdef class DNSOutgoing:

    cdef public unsigned int flags
    cdef public object finished
    cdef public object id
    cdef public bint multicast
    cdef public cython.list packets_data
    cdef public object names
    cdef public cython.list data
    cdef public unsigned int size
    cdef public object allow_long
    cdef public object state
    cdef public cython.list questions
    cdef public cython.list answers
    cdef public cython.list authorities
    cdef public cython.list additionals

    cdef _reset_for_next_packet(self)

    cdef _write_byte(self, object value)

    cdef _insert_short_at_start(self, object value)

    cdef _replace_short(self, object index, object value)

    cdef _write_int(self, object value)

    cdef _write_question(self, object question)

    cdef _write_record_class(self, object record)

    cdef _check_data_limit_or_rollback(self, object start_data_length, object start_size)

    cdef _write_questions_from_offset(self, object questions_offset)

    cdef _write_answers_from_offset(self, object answer_offset)

    cdef _write_records_from_offset(self, object records, object offset)

    cdef _has_more_to_add(self, object questions_offset, object answer_offset, object authority_offset, object additional_offset)

    @cython.locals(
        questions_offset=cython.uint,
        answer_offset=cython.uint,
        authority_offset=cython.uint,
        additional_offset=cython.uint,
        questions_written=cython.uint,
        answers_written=cython.uint,
        authorities_written=cython.uint,
        additionals_written=cython.uint,
    )
    cdef _packets(self)
